﻿#Region " Imports "

Imports System.Globalization

#End Region

<Assembly: CLSCompliant(True)> 

Public Module Main

#Region " Main Subroutine "

    Public Sub Main()
        Try
            Dim folders As ObjectModel.ReadOnlyCollection(Of String) = My.Computer.FileSystem.GetDirectories(My.Computer.FileSystem.CurrentDirectory)
            Dim foldersNUS As New Generic.List(Of String)

            ' Get NUS valid folders only
            For Each folder As String In folders
                If folder.IsNusFolder Then
                    foldersNUS.Add(folder)
                End If
            Next

            ' Create folders
            For Each folder As String In foldersNUS
                Call CreateFolders(folder, True)
            Next

        Catch ex As Exception
            Console.WriteLine(ex.Message)
            Console.WriteLine("Press any key to exit.")
            Console.ReadKey()
        End Try
    End Sub

#End Region

#Region " Methods "

    ''' <summary>
    ''' Create folders using NUS valid folder
    ''' </summary>
    ''' <param name="folderName">NUS valid folder</param>
    ''' <param name="copyFiles">Boolean value indicating to copy (True) or move (False) the files</param>
    ''' <remarks></remarks>
    Private Sub CreateFolders(ByVal folderName As String, ByVal copyFiles As Boolean)
        Dim folder As String = My.Computer.FileSystem.GetName(folderName).ToLower(CultureInfo.CurrentCulture)
        Dim part1 As String = folder.Substring(0, 8)
        Dim part2 As String = folder.Substring(8, 8)
        Dim part3 As String = folder.Substring(16)
        Dim currentFolder As String = My.Computer.FileSystem.CurrentDirectory

        ' Creating the first folder
        My.Computer.FileSystem.CreateDirectory(part1)

        ' Creating the second folder
        My.Computer.FileSystem.CurrentDirectory = My.Computer.FileSystem.CombinePath(My.Computer.FileSystem.CurrentDirectory, part1)
        My.Computer.FileSystem.CreateDirectory(part2)

        ' Creating the third folder
        My.Computer.FileSystem.CurrentDirectory = My.Computer.FileSystem.CombinePath(My.Computer.FileSystem.CurrentDirectory, part2)
        My.Computer.FileSystem.CreateDirectory(part3)

        ' Setting the current directory
        My.Computer.FileSystem.CurrentDirectory = currentFolder

        ' Copying / Moving files
        Dim destination As String = String.Format(CultureInfo.CurrentCulture, "{0}\{1}\{2}\{3}", currentFolder, part1, part2, part3)

        If copyFiles Then
            My.Computer.FileSystem.CopyDirectory(folderName, destination, True)
        Else
            My.Computer.FileSystem.MoveDirectory(folderName, destination, True)
        End If
    End Sub

#End Region

End Module

#Region " Extension Methods "

Public Module StringExtensions

    ''' <summary>
    ''' Returns a Boolean value indicating whether a folder represents a valid NUS folder
    ''' </summary>
    ''' <param name="folderName">Folder Name</param>
    ''' <returns>
    ''' Returns True if Path is a valid NUS folder; otherwise, it returns False
    ''' </returns>
    ''' <remarks></remarks>
    <System.Runtime.CompilerServices.Extension()> Public Function IsNusFolder(ByVal folderName As String) As Boolean
        Dim folder As String = My.Computer.FileSystem.GetName(folderName).ToLower(CultureInfo.CurrentCulture)

        If folder.Length >= 18 Then
            Dim part1 As String = folder.Substring(0, 8)
            Dim part2 As String = folder.Substring(8, 8)
            Dim part3 As String = folder.Substring(16, 1) ' v
            Dim part4 As String = folder.Substring(17) ' version number

            Return (IsHex(part1) And IsHex(part2) And part3 = "v" And IsNumeric(part4))
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Returns a Boolean value indicating whether an expression represents a valid hexadecimal value
    ''' </summary>
    ''' <param name="expression">String expression</param>
    ''' <returns>
    ''' Returns True if String Expression represents a valid hexadecimal value; otherwise, it returns False
    ''' </returns>
    ''' <remarks></remarks>
    Private Function IsHex(ByVal expression As String) As Boolean
        If expression Is Nothing OrElse expression.Length = 0 Then
            Return False
        Else
            Dim currentChar As Char

            For Each currentChar In expression.ToCharArray
                If Not ((currentChar >= "0"c AndAlso currentChar <= "9"c) OrElse (currentChar >= "a"c AndAlso currentChar <= "f") OrElse (currentChar >= "A"c AndAlso currentChar <= "F"c)) Then
                    Return False
                End If
            Next

            Return True
        End If
    End Function

    ''' <summary>
    ''' Returns a Boolean value indicating whether an expression represents a valid numeric value
    ''' </summary>
    ''' <param name="expression">String Expression</param>
    ''' <returns>
    ''' Returns True if String Expression represents a valid numeric value; otherwise, it returns False
    ''' </returns>
    ''' <remarks></remarks>
    Private Function IsNumeric(ByVal expression As String) As Boolean
        If expression Is Nothing OrElse expression.Length = 0 Then
            Return False
        Else
            Dim result As Double = New Double

            Return Double.TryParse(expression, Globalization.NumberStyles.Any, Globalization.CultureInfo.CurrentCulture, result)
        End If
    End Function

End Module

#End Region
